#include <libglademm/xml.h>
#include <gtkmm.h>
#include <iostream>
#include <vector>

using namespace std;

#include <boost/shared_ptr.hpp>

using namespace boost;

#include "TreeViewObject.h"
#include "TreeViewBase.h"
#include "ListViewBase.h"
#include "TreeViewDerived.h"
#include "ListViewDerived.h"

#include "../AppData.h"

/////////////////////////////////////////////////////////////////////

extern AppData data;

/////////////////////////////////////////////////////////////////////

TreeViewObject::TreeViewObject(GtkTreeView* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade) : Gtk::TreeView(cobject)
{
}

/////////////////////////////////////////////////////////////////////

void TreeViewObject::init( const vector<vector<ColumnType> >& columnSpec )
{
  m_columnSpec = columnSpec;

  unsigned numViewColumns = columnSpec.size();
  unsigned u0,u1;
  Gtk::TreeModelColumnBase* p;

  for ( u0 = 0; u0 < numViewColumns; u0++ ) {
    m_columnTypes.push_back( vector<shared_ptr<Gtk::TreeModelColumnBase> >() );
    for ( u1 = 0; u1 < columnSpec[u0].size(); u1++ ) {
      switch ( columnSpec[u0][u1] ) {
        case ColTypePixbuf : p = new Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> >(); break;
        case ColTypeText   : p = new Gtk::TreeModelColumn<Glib::ustring>(); break;
        case ColTypeToggle : p = new Gtk::TreeModelColumn<bool>(); break;
      }
      m_columnTypes.back().push_back( shared_ptr<Gtk::TreeModelColumnBase>(p) );
      m_columnRecord.add( *p );
    }
  }

  Glib::RefPtr<Gtk::TreeModel> model = create();
  set_model( model );

  Gtk::TreeViewColumn* q;
  Gtk::CellRenderer* r;

  for ( u0 = 0; u0 < numViewColumns; u0++ ) {
    q = new Gtk::TreeViewColumn;
    m_columns.push_back( shared_ptr<Gtk::TreeViewColumn>( q ) );
    for ( u1 = 0; u1 < columnSpec[u0].size(); u1++ ) {
      switch ( columnSpec[u0][u1] ) {
        case ColTypePixbuf : r = &m_cellRendererPixbuf; break;
        case ColTypeText   : r = &m_cellRendererText; break;
        case ColTypeToggle : r = &m_cellRendererToggle; break;
      }
      q->pack_start( *r, false );
      q->set_renderer( *r, *m_columnTypes[u0][u1] );
    }
    append_column( *q );
  }

}

/////////////////////////////////////////////////////////////////////

void TreeViewObject::setToggle( Gtk::TreeIter& iter, unsigned viewCol, unsigned index, bool value )
{
  iter->set_value( m_columnTypes[viewCol][index]->index(), value );
}

void TreeViewObject::setText( Gtk::TreeIter& iter, unsigned viewCol, unsigned index, const Glib::ustring& value )
{
  iter->set_value( m_columnTypes[viewCol][index]->index(), value );
}

void TreeViewObject::setPixbuf( Gtk::TreeIter& iter, unsigned viewCol, unsigned index, const Glib::RefPtr<Gdk::Pixbuf>& value )
{
  iter->set_value( m_columnTypes[viewCol][index]->index(), value );
}

void TreeViewObject::setImage( Gtk::TreeIter& iter, unsigned viewCol, unsigned index, const string& fileName )
{
  Glib::RefPtr<Gdk::Pixbuf> value = Gdk::Pixbuf::create_from_file( fileName );
  iter->set_value( m_columnTypes[viewCol][index]->index(), value );
}

Gtk::TreeIter TreeViewObject::getRoot()
{
  return( get_model()->get_iter( "0" ) );
}

/////////////////////////////////////////////////////////////////////

